/*  Copyright 1995, Richard M. Troth, all rights reserved.  <plaintext>
 *
 *        Name: UFTXDSPL REXX (CMS Pipelines "gem")
 *              de-spool a file into a SIFT job (UFT usable format)
 *      Author: Rick Troth, Houston, Texas, USA
 *        Date: 1995-Jan-25
 *
 *        Note: UFTXDSPL is not a user-level pipeline stage.
 */

/*  identify this stage to itself  */
Parse Source . . arg0 .
argo = arg0 || ':'
argl = '*' || argo

uc = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
lc = "abcdefghijklmnopqrstuvwxyz"

/*  parse command line for spool file ID number */
Parse Arg sid . '(' . ')' .

/*  does the file exist?  */
Parse Value Diagrc(08,'QUERY READER' sid) With ,
    1 rc 10 . 17 rs '15'x rdr '15'x .
If rc ^= 0 Then Do
    Say rs
    Exit rc
    End  /*  If .. Do  */

/*  now parse the response from CP  */
Parse Var rdr 1 from 9 . 15 class 16 . 17 devtype 20 . ,
        21 records 29 . 30 cpy 33 . 34 hold 38 . ,
        39 form 47 . 48 dest 56 . 57 keep 61 . 62 msg 65 .
from = Translate(Strip(from),lc,uc)

/*  also get the "table" data  */
Parse Value Diagrc(08,'QUERY READER' sid 'TBL') With ,
    1 rc 10 . 17 rs '15'x tbl '15'x .
If rc ^= 0 Then Do
    Say rs
    Exit rc
    End  /*  If .. Do  */

/*  and parse that  */
Parse Var tbl 1 . ,
        30 flash 35 . 36 fcb 39 . 40 mdfy 45 . ,
        46 flshc 51 . 52 load 56 . 57 chars . 77 size 81 .
If Datatype(size,'W') Then size = (size * 4) || "K"

Parse Value Diagrc(08,'QUERY READER' sid 'ALL') With ,
    1 rc 10 . 17 rs '15'x info '15'x .
If rc ^= 0 Then Do
    Say rs
    Exit rc
    End  /*  If .. Do  */
Parse Var info 1 . ,
        39 date 44 . 45 time 53 . 54 name 62 . ,
            64 type 72 . 73 dist 81 .
name = Strip(name);  type = Strip(type)
If name ^= "" | type ^= "" Then name = name || "." || type
                           Else name = ""
name = Translate(name,lc,uc)

Address CMS 'GETFMADR'
If rc ^= 0 Then Exit rc
Parse Pull . . va .
Parse Value Diagrc(08,'DEFINE READER' va) With 1 rc 10 . 17 rs '15'x .
If rc ^= 0 Then Do
    Say rs
    Exit rc
    End  /*  If .. Do  */
Call Diag 08, 'SPOOL' va 'HOLD CLASS *'

/*  order this file to top-of-queue  */
Parse Value Diagrc(08,'ORDER READER' sid) With 1 rc 10 . 17 rs '15'x .
If rc ^= 0 Then Do
    Say rs
    Call Diag 08, 'DETACH' va
    Exit rc
    End  /*  If .. Do  */

/*  attach reader stage to input  */
'ADDPIPE READER' va 'FILE' sid '| *.INPUT:'
If rc ^= 0 Then Do;  Call Diag 08, 'DETACH' va;  Exit rc;  End

/*  examine the TAG record (should be a NOP CCW)  */
'READTO TAG'
Parse Var tag 1 _cc 2 tag
If rc ^= 0 Then Do;  Call Diag 08, 'DETACH' va;  Exit rc;  End

/*  examine second record,  first real record  */
'PEEKTO RECORD'
If rc ^= 0 Then Do;  Call Diag 08, 'DETACH' va;  Exit rc;  End
Parse Var record 1 _cc 2 data

/*  output the SIFT job header records  */
'OUTPUT' "FILE" size from "AGENT"
'OUTPUT' argl "SPOOLID" sid "OWNER" Userid()
tag = Strip(tag)
If tag ^= "" Then 'OUTPUT' "*TAG" tag

cc = ""
Select
    When type = "MAIL" | type = "NOTE"  Then    Call XMAILUFT
    When Left(data,4) = "*UFT"          Then    Call READYUFT
    When Substr(data,3,4) = "INMR"      Then    Call NDATAUFT
    Otherwise                                   Call SPOOLUFT
    End  /*  Select  */

/*  output the SIFT job header records  */
'OUTPUT' "TYPE" type cc
If name ^= "" Then 'OUTPUT' "NAME" name

'OUTPUT' "CLASS" class devtype
'OUTPUT' "COPIES" cpy
'OUTPUT' "HOLD" hold
'OUTPUT' "FORM" form
'OUTPUT' "DEST" dest
'OUTPUT' "DIST" dist
'OUTPUT' "KEEP" keep
'OUTPUT' "MSG" msg

/*  pass the file contents  */
'OUTPUT' "DATA"
'CALLPIPE *: | *:'

'SEVER INPUT'
Call Diag 08, 'CLOSE' va
Call Diag 08, 'DETACH' va

Exit

/* ------------------------------------------------------------ READYUFT
 *  Say "Looks like a UFT-ready file."
 */
READYUFT:
_dt = ""        /*  sender's DEVTYPE  */
Do Forever
    'PEEKTO RECORD'
    If rc ^= 0 Then Leave
    Parse Var record 1 _cc 2 data
    If _cc ^= '03'x Then Leave
    Parse Upper Var data verb .
    Select  /*  verb  */
        When Abbrev("FILE",verb,1)    Then nop
        When Abbrev("USER",verb,1)    Then nop
        When Abbrev("TYPE",verb,1)    Then Parse Var data . type cc .
        When Abbrev("NAME",verb,1)    Then Parse Var data . name .
        When Abbrev("CLASS",verb,2)   Then Parse Var data . class _dt .
        When Abbrev("COPIES",verb,2)  Then Parse Var data . cpy .
        When Abbrev("HOLD",verb,2)    Then Parse Var data . hold .
        When Abbrev("FORM",verb,2)    Then Parse Var data . form .
        When Abbrev("DEST",verb,2)    Then Parse Var data . dest .
        When Abbrev("DIST",verb,2)    Then Parse Var data . dist .
        When Abbrev("KEEP",verb,4)    Then Parse Var data . keep .
        When Abbrev("MSG",verb,3)     Then Parse Var data . msg .
        When Abbrev("DATA",verb,3)    Then nop
        Otherwise 'OUTPUT' data
        End  /*  Select verb  */
    'READTO'
    If rc ^= 0 Then Leave
    End  /*  Do Forever  */
/*  did sender provide a DEVTYPE?  */
If _dt ^= "" Then devtype = _dt
/*  if it looks like a VM-to-VM transfer,  then   */
If type = "V" & cc = "M" Then Return
/*  else,  remove CCWs from stream,  and possibly pad  */
'ADDPIPE *.INPUT: | NLOCATE 1.1' '000300'x '| SPEC 2-* 1 | *.INPUT:'
If rc ^= 0 Then Do;  Call Diag 08, 'DETACH' va;  Exit rc;  End
If devtype = "PUN" Then 'ADDPIPE *.INPUT: | PAD 80 | *.INPUT:'
Return

/* ------------------------------------------------------------ NDATAUFT
 *  Say "Looks like a NETDATA file."
 */
NDATAUFT:
type = "N"
/*  remove CCWs from stream,  and possibly pad  */
'ADDPIPE *.INPUT: | NLOCATE 1.1' '000300'x '| SPEC 2-* 1 | *.INPUT:'
If rc ^= 0 Then Do;  Call Diag 08, 'DETACH' va;  Exit rc;  End
If devtype = "PUN" Then 'ADDPIPE *.INPUT: | PAD 80 | *.INPUT:'
Return

/* ------------------------------------------------------------ SPOOLUFT
 *  Say "Looks like an ordinary spool file."
 */
SPOOLUFT:
/*  variable length records with machine carriage control  */
type = "V M"
Return

/* ------------------------------------------------------------ XMAILUFT
 *  From the filetype,  this should be  "mail".
 */
XMAILUFT:
type = "M"
/*  remove CCWs from stream,  and check for RFC 822 header  */
'ADDPIPE *.INPUT: | NLOCATE 1.1' '000300'x '| SPEC 2-* 1 | *.INPUT:'
If rc ^= 0 Then Do;  Call Diag 08, 'DETACH' va;  Exit rc;  End
'PEEKTO RECORD'
Return


